# coding=utf-8
import pathlib
from nwpc_hpc_model.workload.record_parser import RecordParser, TableRecordParser


class DetailLabelParser(RecordParser):
    def __init__(self, label):
        RecordParser.__init__(self)

        self.label = label

    def __deepcopy__(self, memodict={}):
        return DetailLabelParser(self.label)

    def parse(self, record):
        for line in record:
            index = line.find(': ')
            if index == -1:
                continue
            label = line[0:index].strip()
            if label != self.label:
                continue
            value = line[index+2:].strip()
            return value
        return ""


class LlqFilePathParser(DetailLabelParser):
    def __init__(self, label):
        DetailLabelParser.__init__(self, label)

    @staticmethod
    def find_initial_working_dir(record):
        parser = DetailLabelParser("Initial Working Dir")
        value = parser.parse(record)
        return value

    def parse(self, record):
        parser = DetailLabelParser(self.label)
        value = parser.parse(record)

        if value.startswith('.'):
            initial_working_dir = LlqFilePathParser.find_initial_working_dir(record)
            value = str(pathlib.PurePosixPath(initial_working_dir, value))

        return value


class LlqJobScriptParser(RecordParser):
    def __init__(self):
        RecordParser.__init__(self)

    @staticmethod
    def find_step_type_value(record):
        parser = DetailLabelParser("Step Type")
        value = parser.parse(record)
        return value

    def parse(self, record):
        step_type = LlqJobScriptParser.find_step_type_value(record)

        if step_type == "General Parallel":
            script_label = "Executable"
        elif step_type == "Serial":
            script_label = "Cmd"
        else:
            return ""

        file_path_parser = LlqFilePathParser(script_label)
        value = file_path_parser.parse(record)
        return value



# coding=utf-8
import datetime


class ValueSaver(object):
    def __init__(self):
        pass

    def set_item_value(self, item, value):
        pass


class StringSaver(ValueSaver):
    def __init__(self):
        ValueSaver.__init__(self)

    def set_item_value(self, item, value):
        item.map['text'] = value
        item.map['value'] = value
        item.map['data'] = value


class NumberSaver(ValueSaver):
    def __init__(self):
        ValueSaver.__init__(self)

    def set_item_value(self, item, value):
        data = float(value)
        item.map['text'] = value
        item.map['value'] = value
        item.map['data'] = data


class DateTimeSaver(ValueSaver):
    def __init__(self, datetime_format):
        ValueSaver.__init__(self)
        self.datetime_format = datetime_format

    def set_item_value(self, item, value):
        data = datetime.datetime.strptime(value, self.datetime_format)

        item.map['text'] = data.strftime("%Y-%m-%d %H:%M:%S")
        item.map['value'] = value
        item.map['data'] = data



# coding: utf-8
from pathlib import Path

from nwpc_hpc_model.workload.slurm import SlurmQueryModel, SlurmQueryCategoryList
from nwpc_hpc_model.workload import QueryCategory, record_parser, value_saver


class TestSinfoQueryInfo(object):

    def test_to_dict(self):
        category_list = SlurmQueryCategoryList()

        category_list.extend([
            QueryCategory("sinfo.partition", "Partition", "PARTITION",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.avail", "Avail", "AVAIL",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.time_limit", "Time Limit", "TIMELIMIT",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.nodes", "Nodes", "NODES",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.NumberSaver, ()),
            QueryCategory("sinfo.state", "State", "STATE",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.node_list", "Node List", "NODELIST",
                          record_parser.TokenRecordParser,  (-1,),
                          value_saver.StringSaver, ()),
        ])

        test_case_list = list()

        nwp_output = Path(
            Path(__file__).parent,
            "../data/sinfo/default.txt"
        )
        with open(nwp_output) as nwp_output_file:
            lines = nwp_output_file.read().split('\n')
            model = SlurmQueryModel.build_from_table_category_list(lines, category_list)
            model_dict = model.to_dict()
            assert(('items' in model_dict) is True)
            assert(len(model_dict['items']) == 12)

    def check_build_model(self, test_case):
        record = test_case['record']
        category_list = test_case['category_list']
        records = test_case['records']

        model = SlurmQueryModel.build_from_table_category_list(record, category_list)

        for record in records:
            index = record['index']
            properties = record['properties']
            item = model.items[index]

            for a_prop in properties:
                category_id = a_prop['category_id']
                text = a_prop['text']
                category_index = category_list.index_from_id(category_id)
                p = item.props[category_index]

                assert(p.map['text'] == text)

    def test_build_model(self):
        check_method = self.check_build_model

        category_list = SlurmQueryCategoryList()

        category_list.extend([
            QueryCategory("sinfo.partition", "Partition", "PARTITION",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.avail", "Avail", "AVAIL",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.time_limit", "Time Limit", "TIMELIMIT",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.nodes", "Nodes", "NODES",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.NumberSaver, ()),
            QueryCategory("sinfo.state", "State", "STATE",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.StringSaver, ()),
            QueryCategory("sinfo.node_list", "Node List", "NODELIST",
                          record_parser.TokenRecordParser, (-1,),
                          value_saver.StringSaver, ()),
        ])

        test_case_list = list()

        nwp_output = Path(
            Path(__file__).parent,
            "../data/sinfo/default.txt"
        )

        with open(nwp_output) as nwp_output_file:
            record = nwp_output_file.readlines()
            test_case_list.append({
                'record': record,
                'category_list': category_list,
                'records': [
                    {
                        'index': 0,
                        'properties': [
                            {
                                'category_id': 'sinfo.partition',
                                'text': 'serial'
                            },
                            {
                                'category_id': 'sinfo.state',
                                'text': 'idle'
                            }
                        ]
                    },
                    {
                        'index': 1,
                        'properties': [
                            {
                                'category_id': 'sinfo.partition',
                                'text': 'normal'
                            },
                            {
                                'category_id': 'sinfo.state',
                                'text': 'drain'
                            }
                        ]
                    }
                ]
            })

        for a_test_case in test_case_list:
            check_method(a_test_case)



# coding=utf-8
from nwpc_hpc_model.workload import QueryModel, QueryItem


class SlurmQueryModel(QueryModel):
    def __init__(self):
        QueryModel.__init__(self)

    @classmethod
    def build_from_table_category_list(cls, record, category_list, sep=None):
        if len(record) == 0:
            return None

        title_line = record[0]
        category_list.update_index_for_table_query(title_line, sep)

        lines = record[1:]
        model = SlurmQueryModel()

        if len(lines) == 0:
            return model

        for line in lines:
            if len(line) == 0:
                continue
            item = QueryItem.build_from_category_list(line, category_list)
            model.items.append(item)

        return model



# coding: utf-8
from nwpc_hpc_model.workload import QueryCategoryList


class SlurmQueryCategoryList(QueryCategoryList):

    def update_index_for_table_query(self, title_line, sep=None):
        title_line = title_line.strip()
        if sep is None:
            tokens = title_line.split()
        else:
            tokens = title_line.split('|')
        for index in range(0, len(tokens)):
            label = tokens[index]
            category = self.category_from_label(label)
            if category:
                category.record_parser_arguments = (index, sep)
                category.build_record_parser()



# coding: utf-8


class QueryModel(object):
    def __init__(self):
        self.items = list()
        self.category_list = None

    def to_dict(self):
        result = dict()
        result['items'] = []
        for item in self.items:
            result['items'].append(item.to_dict())
        return result

    def set_category_list(self, category_list):
        self.category_list = category_list



# coding=utf-8
from nwpc_hpc_model.workload import QueryModel
from . import QueryItem


class LoadLevelerQueryModel(QueryModel):
    def __init__(self):
        QueryModel.__init__(self)

    @classmethod
    def build_from_category_list(cls, record, category_list):
        lines = record
        model = QueryModel()

        if lines[0].startswith('llq: There is currently no job status to report.'):
            # not jobs
            return model

        if lines[0].startswith('llq:'):
            print('failure detected:', lines)
            return None
        if len(lines) < 3:
            print('unsupported output:', lines)
            return None

        summary_line = lines[-2]

        record_start_line_no_list = list()

        for i in range(0, len(lines) - 3):
            if lines[i].startswith("====="):
                record_start_line_no_list.append(i)
        record_start_line_no_list.append(len(lines)-2)

        for record_no in range(0, len(record_start_line_no_list)-1):
            record_lines = lines[record_start_line_no_list[record_no]: record_start_line_no_list[record_no+1]]

            item = QueryItem.build_from_category_list(record_lines, category_list)

            model.items.append(item)

        return model



# coding=utf-8
import unittest
import os

from nwpc_hpc_model.loadleveler import query_model, query_category, record_parser, value_saver


class TestLlclassDetailQueryModel(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def check_build_model(self, test_case):
        record = test_case['record']
        category_list = test_case['category_list']
        records = test_case['records']

        model = query_model.LoadLevelerQueryModel.build_from_category_list(record, category_list)

        for record in records:
            index = record['index']
            properties = record['properties']
            item = model.items[index]

            for a_prop in properties:
                category_id = a_prop['category_id']
                text = a_prop['text']
                category_index = category_list.index_from_id(category_id)
                p = item.props[category_index]

                self.assertEqual(p.map['text'], text)

    def test_build_model(self):
        check_method = self.check_build_model

        category_list = query_category.QueryCategoryList()

        category_list.extend([
            query_category.QueryCategory("llclass.name", "Name", "Name",
                                         record_parser.DetailLabelParser, ("Name",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llclass.free_slots", "Free Slots", "Free_slots",
                                         record_parser.DetailLabelParser, ("Free_slots",),
                                         value_saver.NumberSaver, ()),
            query_category.QueryCategory("llclass.maximum_slots", "Maximum Slots", "Maximum_slots",
                                         record_parser.DetailLabelParser, ("Maximum_slots",),
                                         value_saver.NumberSaver, ())
        ])

        test_case_list = list()

        nwp_output = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llclass/operation.txt"
        )
        with open(nwp_output) as nwp_output_file:
            record = nwp_output_file.readlines()
            test_case_list.append({
                'record': record,
                'category_list': category_list,
                'records': [
                    {
                        'index': 0,
                        'properties': [
                            {
                                'category_id': 'llclass.name',
                                'text': 'operation'
                            },
                            {
                                'category_id': 'llclass.free_slots',
                                'text': '384'
                            },
                            {
                                'category_id': 'llclass.maximum_slots',
                                'text': '2560'
                            }
                        ]
                    }
                ]
            })

        for a_test_case in test_case_list:
            check_method(a_test_case)



# coding=utf-8
from nwpc_hpc_model.workload import query_category, query_item, query_property
from nwpc_hpc_model.workload.query_property import QueryProperty
from nwpc_hpc_model.workload.query_item import QueryItem
from nwpc_hpc_model.workload.query_category import QueryCategory, QueryCategoryList
from .query_model import LoadLevelerQueryModel



import datetime
from nwpc_hpc_model.loadleveler.filter_condition import \
    FilterCondition, \
    get_property_data, \
    PropertyFilterCondition, \
    create_equal_value_checker, \
    create_greater_value_checker, \
    create_less_value_checker, \
    create_in_value_checker, \
    create_value_in_checker

from nwpc_hpc_model.loadleveler import QueryItem, QueryProperty


def create_job_item_dict(
        job_id="id_no",
        owner="owner",
        job_class="job_class",
        queue_date=datetime.datetime.now(),
        status="R",
        priority=100,
        job_script="/script_path"
):
    return {
        "props": [
            {
                "id": "llq.id",
                "data": job_id,
                "text": job_id,
                "value": job_id
            },
            {
                "id": "llq.owner",
                "data": owner,
                "text": owner,
                "value": owner
            },
            {
                "id": "llq.class",
                "data": job_class,
                "text": job_class,
                "value": job_class
            },
            {
                "id": "llq.status" + job_id,
                "data": status,
                "text": status,
                "value": status
            },
            {
                "id": "llq.queue_date",
                "data": queue_date.strftime("%Y-%m-%d %H:%M:%S"),  # 2017-04-21 07:08:43
                "text": queue_date.strftime("%m/%d %H:%M"),  # "04/21 07:08",
                "value": queue_date.strftime("%a %b %d %H:%M:%S %Y"),  # "Fri Apr 21 07:08:43 2017"
            },
            {
                "id": "llq.priority",
                "data": priority,
                "text": priority,
                "value": priority
            },
            {
                "id": "llq.job_script",
                "data": job_script,
                "text": job_script,
                "value": job_script
            },
        ]
    }


def create_job_item(
        job_id="id_no",
        owner="owner",
        job_class="job_class",
        queue_date=datetime.datetime.now(),
        status="R",
        priority=100,
        job_script="/script_path"
):
    job_item = QueryItem()

    query_property = QueryProperty()
    query_property.map['id'] = "llq.job_id"
    query_property.map['text'] = job_id
    query_property.map['value'] = job_id
    query_property.map['data'] = job_id
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.owner"
    query_property.map['text'] = owner
    query_property.map['value'] = owner
    query_property.map['data'] = owner
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.class"
    query_property.map['text'] = job_class
    query_property.map['value'] = job_class
    query_property.map['data'] = job_class
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.queue_date"
    query_property.map['text'] = queue_date
    query_property.map['value'] = queue_date
    query_property.map['data'] = queue_date
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.status"
    query_property.map['text'] = status
    query_property.map['value'] = status
    query_property.map['data'] = status
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.status"
    query_property.map['text'] = str(priority)
    query_property.map['value'] = priority
    query_property.map['data'] = priority
    job_item.props.append(query_property)

    query_property = QueryProperty()
    query_property.map['id'] = "llq.job_script"
    query_property.map['text'] = job_script
    query_property.map['value'] = job_script
    query_property.map['data'] = job_script
    job_item.props.append(query_property)

    return job_item


def test_get_property_data():
    job_item = create_job_item_dict(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=50,
        job_script="/cma/g3/test.ksh"
    )

    assert get_property_data(job_item, "llq.job_script") == "/cma/g3/test.ksh"

    job_item = create_job_item(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=50,
        job_script="/cma/g3/test.ksh"
    )

    assert get_property_data(job_item, "llq.job_script") == "/cma/g3/test.ksh"


def test_condition():
    condition = FilterCondition()
    job_item = dict()
    assert condition.is_fit(job_item)


def test_equal_value_checker():
    job_item = create_job_item_dict(job_id="id", owner="nwp_xp", job_class="serial", queue_date=datetime.datetime.now())

    condition = PropertyFilterCondition(
        property_id="llq.owner",
        data_checker=create_equal_value_checker("nwp_xp"))

    assert condition.is_fit(job_item)

    condition = PropertyFilterCondition(
        property_id="llq.owner",
        data_checker=create_equal_value_checker("unknown")
    )
    assert not condition.is_fit(job_item)


def test_greater_value_checker():
    job_item = create_job_item_dict(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=100
    )
    condition = PropertyFilterCondition(
        property_id="llq.priority",
        data_checker=create_greater_value_checker(50))

    assert condition.is_fit(job_item)

    condition = PropertyFilterCondition(
        property_id="llq.priority",
        data_checker=create_greater_value_checker(100)
    )
    assert not condition.is_fit(job_item)


def test_less_value_checker():
    job_item = create_job_item_dict(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=50
    )
    condition = PropertyFilterCondition(
        property_id="llq.priority",
        data_checker=create_less_value_checker(100))

    assert condition.is_fit(job_item)

    condition = PropertyFilterCondition(
        property_id="llq.priority",
        data_checker=create_less_value_checker(50)
    )
    assert not condition.is_fit(job_item)


def test_in_value_checker():
    job_item = create_job_item_dict(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=50
    )
    condition = PropertyFilterCondition(
        property_id="llq.owner",
        data_checker=create_in_value_checker(("nwp_xp", "nwp", "nwp_qu")))

    assert condition.is_fit(job_item)

    condition = PropertyFilterCondition(
        property_id="llq.owner",
        data_checker=create_in_value_checker(("unknown",))
    )
    assert not condition.is_fit(job_item)


def test_value_in_checker():
    job_item = create_job_item_dict(
        job_id="id",
        owner="nwp_xp",
        job_class="serial",
        queue_date=datetime.datetime.now(),
        priority=50,
        job_script="/cma/g3/test.ksh"
    )
    condition = PropertyFilterCondition(
        property_id="llq.job_script",
        data_checker=create_value_in_checker("g3")
    )

    assert condition.is_fit(job_item)

    condition = PropertyFilterCondition(
        property_id="llq.job_script",
        data_checker=create_value_in_checker("g2")
    )
    assert not condition.is_fit(job_item)



import unittest
from unittest import mock
import os
import importlib
import datetime

from nwpc_hpc_model.loadleveler import value_saver
from nwpc_hpc_model.loadleveler import QueryProperty


class TestValueSaver(unittest.TestCase):
    def setUp(self):
        importlib.reload(value_saver)

    def tearDown(self):
        pass

    def check_string(self, test_case):
        name = test_case['name']
        value = test_case['value']
        text = test_case['text']
        data = test_case['data']

        saver = value_saver.StringSaver()
        item = QueryProperty()

        saver.set_item_value(item, value)

        self.assertEqual(item.map['text'], text)
        self.assertEqual(item.map['data'], data)
        print("Test passed:", name)

    def test_string(self):
        test_method = self.check_string
        test_case_list = [
            {
                'name': 'id',
                'value': 'cma20n02.2871950.0',
                'text': 'cma20n02.2871950.0',
                'data': 'cma20n02.2871950.0'
            },
            {
                'name': 'owner',
                'value': 'nwp_vfy',
                'text': 'nwp_vfy',
                'data': 'nwp_vfy'
            },
            {
                'name': 'class',
                'value': 'serial',
                'text': 'serial',
                'data': 'serial'
            },
        ]

        for a_test_case in test_case_list:
            test_method(a_test_case)

    def check_number(self, test_case):
        name = test_case['name']
        value = test_case['value']
        text = test_case['text']
        data = test_case['data']

        saver = value_saver.NumberSaver()
        item = QueryProperty()

        saver.set_item_value(item, value)

        self.assertEqual(item.map['text'], text)
        self.assertEqual(item.map['data'], data)
        print("Test passed:", name)

    def test_number(self):
        test_method = self.check_number
        test_case_list = [
            {
                'name': 'PRI',
                'value': '100',
                'text': '100',
                'data': 100
            }
        ]

        for a_test_case in test_case_list:
            test_method(a_test_case)

    def check_full_date(self, test_case):
        name = test_case['name']
        value = test_case['value']
        text = test_case['text']
        data = test_case['data']

        saver = value_saver.FullDateSaver()
        item = QueryProperty()

        saver.set_item_value(item, value)

        self.assertEqual(item.map['text'], text)
        self.assertEqual(item.map['data'], data)
        print("Test passed:", name)

    def test_full_date(self):
        test_method = self.check_full_date
        test_case_list = [
            {
                'name': '0',
                'value': 'Wed Sep  7 07:00:02 2016',
                'text': '09/07 07:00',
                'data': datetime.datetime.strptime("2016/09/07 07:00:02", "%Y/%m/%d %H:%M:%S")
            },
            {
                'name': '1',
                'value': 'Tue Aug 23 01:54:30 2016',
                'text': '08/23 01:54',
                'data': datetime.datetime.strptime("2016/08/23 01:54:30", "%Y/%m/%d %H:%M:%S")
            }
        ]

        for a_test_case in test_case_list:
            test_method(a_test_case)

    def check_job_state(self, test_case):
        name = test_case['name']
        value = test_case['value']
        text = test_case['text']
        data = test_case['data']

        saver = value_saver.JobStatusSaver()
        item = QueryProperty()

        saver.set_item_value(item, value)

        self.assertEqual(item.map['value'], value)
        self.assertEqual(item.map['text'], text)
        self.assertEqual(item.map['data'], data)
        print("Test passed:", name)

    def test_job_state(self):
        test_method = self.check_job_state
        test_case_list = [
            {
                'name': 'R',
                'value': 'R',
                'text': 'R',
                'data': 'R'
            },
            {
                'name': 'Running',
                'value': 'Running',
                'text': 'R',
                'data': 'R'
            },
            {
                'name': 'RP',
                'value': 'RP',
                'text': 'RP',
                'data': 'RP'
            },
            {
                'name': 'Remove Pending',
                'value': 'Remove Pending',
                'text': 'RP',
                'data': 'RP'
            },

        ]

        for a_test_case in test_case_list:
            test_method(a_test_case)



# coding=utf-8
import datetime
from nwpc_hpc_model.workload.value_saver import ValueSaver, StringSaver, NumberSaver


class FullDateSaver(ValueSaver):
    def __init__(self):
        ValueSaver.__init__(self)

    def set_item_value(self, item, value):
        data = datetime.datetime.strptime(value, "%c")

        item.map['text'] = data.strftime("%m/%d %H:%M")
        item.map['value'] = value
        item.map['data'] = data


job_state_list = [
    {"name": "Canceled",           "abbreviation": "CA"},
    {"name": "Checkpointing",      "abbreviation": "CK"},
    {"name": "Completed",          "abbreviation": "C"},
    {"name": "Complete Pending",   "abbreviation": "CP"},
    {"name": "Deferred",           "abbreviation": "D"},
    {"name": "Idle",               "abbreviation": "I"},
    {"name": "Not Queued",         "abbreviation": "NQ"},
    {"name": "Not Run",            "abbreviation": "NR"},
    {"name": "Pending",            "abbreviation": "P"},
    {"name": "Preempted",          "abbreviation": "E"},
    {"name": "Preempt Pending",    "abbreviation": "EP"},
    {"name": "Rejected",           "abbreviation": "X"},
    {"name": "Reject Pending",     "abbreviation": "XP"},
    {"name": "Removed",            "abbreviation": "RM"},
    {"name": "Remove Pending",     "abbreviation": "RP"},
    {"name": "Resume Pending",     "abbreviation": "MP"},
    {"name": "Running",            "abbreviation": "R"},
    {"name": "Starting",           "abbreviation": "ST"},
    {"name": "System Hold",        "abbreviation": "S"},
    {"name": "Terminated",         "abbreviation": "TX"},
    {"name": "User & System Hold", "abbreviation": "HS"},
    {"name": "User Hold",          "abbreviation": "H"},
    {"name": "Vacated",            "abbreviation": "V"},
    {"name": "Vacate Pending",     "abbreviation": "VP"}
]


class JobStatusSaver(ValueSaver):
    def __init__(self):
        ValueSaver.__init__(self)

    def set_item_value(self, item, value):
        value_length = len(value)
        if value_length == 1 or value_length == 2:
            item.map['text'] = value
            item.map['value'] = value
            item.map['data'] = value
            return

        for a_job_state in job_state_list:
            if a_job_state['name'] == value:
                item.map['text'] = a_job_state['abbreviation']
                item.map['value'] = value
                item.map['data'] = a_job_state['abbreviation']
                return

        item.map['text'] = value
        item.map['value'] = value
        item.map['data'] = value
        return



# coding=utf-8

from nwpc_hpc_model.workload import record_parser


class TestRecordParser(object):
    def test_default_token_parser(self):
        line = "operation    up   infinite     26  drain cmac[0085-0086,0092,0109-0110,0707-0722,0775,0808,0961,1301,1369]"
        test_cases = [
            {
                "name": "sinfo.default_query.simple.1",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 0
                    }
                },
                "value": "operation"
            },
            {
                "name": "sinfo.default_query.simple.2",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 2
                    }
                },
                "value": "infinite"
            },
            {
                "name": "sinfo.default_query.simple.3",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 4
                    }
                },
                "value": "drain"
            },
            {
                "name": "sinfo.default_query.simple.3",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 5
                    }
                },
                "value": "cmac[0085-0086,0092,0109-0110,0707-0722,0775,0808,0961,1301,1369]"
            }
        ]

        def check_parser(case):
            case_line = case["line"]
            value = case["value"]
            name = case["name"]
            record_parser_args = case["record_parser"]["args"]

            parser = record_parser.TokenRecordParser(**record_parser_args)
            parser_value = parser.parse(case_line)
            assert(parser_value == value)

        for test_case in test_cases:
            check_parser(test_case)

    def test_custom_token_parser(self):
        line = "nwp_xp|(null)|1|0|NONE"
        test_cases = [
            {
                "name": "sinfo.default_query.simple.1",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 0,
                        "sep": '|'
                    }
                },
                "value": "nwp_xp"
            },
            {
                "name": "sinfo.default_query.simple.2",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 1,
                        "sep": '|'
                    }
                },
                "value": "(null)"
            },
            {
                "name": "sinfo.default_query.simple.3",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 2,
                        "sep": '|'
                    }
                },
                "value": "1"
            },
            {
                "name": "sinfo.default_query.simple.3",
                "line": line,
                "record_parser": {
                    "args": {
                        "index": 3,
                        "sep": '|'
                    }
                },
                "value": "0"
            }
        ]

        def check_parser(case):
            case_line = case["line"]
            value = case["value"]
            name = case["name"]
            record_parser_args = case["record_parser"]["args"]

            parser = record_parser.TokenRecordParser(**record_parser_args)
            parser_value = parser.parse(case_line)
            assert (parser_value == value)

        for test_case in test_cases:
            check_parser(test_case)


import pytest
import copy

from nwpc_hpc_model.loadleveler import query_category
from nwpc_hpc_model.loadleveler import record_parser
from nwpc_hpc_model.loadleveler import value_saver


class TestQueryCategory:
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_deep_copy(self):
        category = query_category.QueryCategory()
        category.id = "llq.id"
        category.record_parser = record_parser.DetailLabelParser("Job Step Id")
        category.value_saver = value_saver.StringSaver()

        new_category = copy.deepcopy(category)
        new_category.id = "llq.submitted"
        new_category.record_parser.label = "Submitted"

        assert category.id != new_category.id
        assert category.record_parser is not new_category.record_parser
        assert category.record_parser.label is not new_category.record_parser.label

    def test_create_query_category(self):
        id_category = query_category.QueryCategory(
            category_id="llq.id",
            display_name="Id",
            label="Job Step Id",
            record_parser_class=record_parser.DetailLabelParser,
            record_parser_arguments=("Job Step Id",),
            value_saver_class=value_saver.StringSaver
        )

        assert id_category.id == "llq.id"
        assert id_category.display_name == "Id"
        assert id_category.label == "Job Step Id"
        assert isinstance(id_category.record_parser, record_parser.DetailLabelParser)
        assert id_category.record_parser.label == "Job Step Id"
        assert isinstance(id_category.value_saver, value_saver.StringSaver)

    def test_llq_detail_category_list(self):
        category_list = query_category.QueryCategoryList()

        category_list.extend([
            query_category.QueryCategory("llq.id",          "Id",           "Job Step Id",
                                         record_parser.DetailLabelParser, ("Job Step Id",),
                                         value_saver.StringSaver,    ()),
            query_category.QueryCategory("llq.owner",       "Owner",        "Owner",
                                         record_parser.DetailLabelParser, ("Owner",),
                                         value_saver.StringSaver,    ()),
            query_category.QueryCategory("llq.class",       "Class",        "Class",
                                         record_parser.DetailLabelParser, ("Class",),
                                         value_saver.StringSaver,    ()),
            query_category.QueryCategory("llq.job_script",  "Job Script",   "Cmd",
                                         record_parser.DetailLabelParser, ("Cmd",),
                                         value_saver.StringSaver,    ()),
            query_category.QueryCategory("llq.status",      "Status",       "Status",
                                         record_parser.DetailLabelParser, ("Status",),
                                         value_saver.JobStatusSaver, ())
        ])

        assert len(category_list) == 5

        assert category_list.index_from_id("llq.id") == 0

        with pytest.raises(ValueError):
            category_list.index_from_id("llclass.id")

        assert category_list.contains_id("llq.id")
        assert not category_list.contains_id("llclass.id")
        assert category_list.category_from_id("llq.id").id == "llq.id"
        assert category_list.category_from_id("llclass.id") is None

        assert category_list.index_from_label("Class") == 2
        assert category_list.contains_label("Class")
        assert category_list.category_from_label("Class").id == "llq.class"



# coding: utf-8
class FilterCondition(object):
    def __init__(self):
        pass

    def is_fit(self, job_item):
        return True


def get_property_data(job_item, property_id):
    """
    get property data from a job item
    :param job_item: QueryItem or a QueryItem dict.
    :param property_id: property_id
    :return: property data
    """
    result = None
    if isinstance(job_item, dict):
        props = job_item['props']
        for a_prop in props:
            if a_prop['id'] == property_id:
                result = a_prop['data']
                return result
    else:
        props = job_item.props
        for a_prop in props:
            if a_prop.map['id'] == property_id:
                result = a_prop.map['data']
                return result
    return result


class PropertyFilterCondition(FilterCondition):
    def __init__(self, property_id, data_checker, data_parser=get_property_data):
        FilterCondition.__init__(self)
        self.property_id = property_id
        self.data_checker = data_checker
        self.data_parser = data_parser

    def is_fit(self, job_item):
        value = self.data_parser(job_item, self.property_id)
        if self.data_checker(value):
            return True
        else:
            return False


def create_equal_value_checker(expect_value):
    def value_checker(value):
        if value == expect_value:
            return True
        else:
            return False

    return value_checker


def create_greater_value_checker(expect_value):
    def value_checker(value):
        if value > expect_value:
            return True
        else:
            return False

    return value_checker


def create_less_value_checker(expect_value):
    def value_checker(value):
        if value < expect_value:
            return True
        else:
            return False

    return value_checker


def create_in_value_checker(expect_values):
    def value_checker(value):
        if value in expect_values:
            return True
        else:
            return False
    return value_checker


def create_value_in_checker(expect_value):
    def value_checker(value):
        if expect_value in value:
            return True
        else:
            return False
    return value_checker



# coding: utf-8
import copy


class QueryCategory(object):
    def __init__(
            self,
            category_id=".valid_id",
            display_name="",
            label="",
            record_parser_class=None,
            record_parser_arguments=tuple(),
            value_saver_class=None,
            value_saver_arguments=tuple()
    ):
        self.id = category_id
        self.display_name = display_name
        self.label = label

        self.record_parser_class = record_parser_class
        self.record_parser_arguments = record_parser_arguments
        self.record_parser = None

        self.value_saver_class = value_saver_class
        self.value_saver_arguments = value_saver_arguments
        self.value_saver = None

        self.build_record_parser()
        self.build_value_saver()

    def __deepcopy__(self, memodict={}):
        new_object = QueryCategory()

        new_object.id = self.id
        new_object.display_name = self.display_name
        new_object.label = self.label

        if self.record_parser:
            new_object.record_parser = copy.deepcopy(self.record_parser)

        new_object.value_saver = self.value_saver
        return new_object

    def build_record_parser(self):
        if self.record_parser_class is not None:
            self.record_parser = self.record_parser_class(*self.record_parser_arguments)

    def build_value_saver(self):
        if self.value_saver_class is not None:
            self.value_saver = self.value_saver_class(*self.value_saver_arguments)


class QueryCategoryList(list):
    def contains_id(self, category_id):
        for a_category in self:
            if a_category.id == category_id:
                return True
        return False

    def index_from_id(self, category_id):
        for i in range(0, len(self)):
            if self[i].id == category_id:
                return i
        raise ValueError("not found")

    def category_from_id(self, category_id):
        for a_category in self:
            if a_category.id == category_id:
                return a_category
        return None

    def contains_label(self, label):
        for a_category in self:
            if a_category.label == label:
                return True
        return False

    def index_from_label(self, label):
        for i in range(0, len(self)):
            if self[i].label == label:
                return i
        return -1

    def category_from_label(self, label):
        for a_category in self:
            if a_category.label == label:
                return a_category
        return None




# coding=utf-8
import unittest
from unittest import mock
import os
import importlib
import copy

from nwpc_hpc_model.loadleveler import record_parser


class TestRecordParser(unittest.TestCase):
    def setUp(self):
        importlib.reload(record_parser)

    def tearDown(self):
        pass

    def test_deep_copy(self):
        parser_one = record_parser.DetailLabelParser("label-1")
        parser_two = copy.deepcopy(parser_one)
        parser_two.label = "label-2"
        self.assertNotEqual(parser_one.label, parser_two.label)

    def check_llq_detail_query_record_parser(self, test_case):
        lines = test_case["lines"]
        label = test_case["label"]
        value = test_case["value"]
        name = test_case["name"]

        parser = record_parser.DetailLabelParser(label)
        parser_value = parser.parse(lines)
        self.assertEqual(parser_value, value)
        print("Test passed:", name)

    def test_llq_detail_query_record_parser(self):
        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        test_case_list = []
        with open(serial_job_running_file_path) as serial_job_running_file:
            lines = serial_job_running_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.serial_job.running.job_step_id",
                    "lines": lines,
                    "label": "Job Step Id",
                    "value": "cma20n04.2882680.0"
                },
                {
                    "name": "llq.serial_job.running.cmd",
                    "lines": lines,
                    "label": "Cmd",
                    "value": "/cma/g1/nwp/SMSOUT/gmf_grapes_gfs_v2_0/grapes_global/12/post/postp_240.job1"
                },
                {
                    "name": "llq.serial_job.running.queue_date",
                    "lines": lines,
                    "label": "Queue Date",
                    "value": "Thu Sep  8 00:09:02 2016"
                },
                {
                    "name": "llq.serial_job.running.cpu_per_core",
                    "lines": lines,
                    "label": "Cpus Per Core",
                    "value": "0"
                },
                {
                    "name": "llq.serial_job.running.task_affinity",
                    "lines": lines,
                    "label": "Task Affinity",
                    "value": ""
                },
            ])

        for a_test_case in test_case_list:
            self.check_llq_detail_query_record_parser(a_test_case)

    def check_llq_script_record_parser(self, test_case):
        lines = test_case["lines"]
        value = test_case["value"]
        name = test_case["name"]

        parser = record_parser.LlqJobScriptParser()
        parser_value = parser.parse(lines)
        self.assertEqual(parser_value, value)
        print("Test passed:", name)

    def test_llq_script_record_parser(self):
        check_method = self.check_llq_script_record_parser
        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        test_case_list = []
        with open(serial_job_running_file_path) as serial_job_running_file:
            lines = serial_job_running_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.serial_job.running.job_script",
                    "lines": lines,
                    "value": "/cma/g1/nwp/SMSOUT/gmf_grapes_gfs_v2_0/grapes_global/12/post/postp_240.job1"
                }
            ])

        parallel_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/parallel_job_running.txt"
        )
        with open(parallel_job_running_file_path) as parallel_job_running_file:
            lines = parallel_job_running_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.parallel_job.running.job_script",
                    "lines": lines,
                    "value": "/cma/g1/nwp_qu/SMSOUT/rafs/cold/00/model/fcst.job1"
                }
            ])

        for a_test_case in test_case_list:
            check_method(a_test_case)

    def check_llq_file_path_record_parser(self, test_case):
        lines = test_case["lines"]
        value = test_case["value"]
        name = test_case["name"]
        label = test_case["label"]

        parser = record_parser.LlqFilePathParser(label)
        parser_value = parser.parse(lines)
        self.assertEqual(parser_value, value)
        print("Test passed:", name)

    def test_llq_file_path_record_parser(self):
        check_method = self.check_llq_file_path_record_parser
        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        test_case_list = []
        with open(serial_job_running_file_path) as serial_job_running_file:
            lines = serial_job_running_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.serial_job.running.out",
                    "lines": lines,
                    "value": "/cma/g1/nwp/SMSOUT/gmf_grapes_gfs_v2_0/grapes_global/12/post/postp_240.1",
                    "label": "Out",
                },
                {
                    "name": "llq.serial_job.running.err",
                    "lines": lines,
                    "value": "/cma/g1/nwp/SMSOUT/gmf_grapes_gfs_v2_0/grapes_global/12/post/postp_240.1.err",
                    "label": "Err",
                }
            ])

        parallel_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/parallel_job_running_3.txt"
        )
        with open(parallel_job_running_file_path) as parallel_job_running_file:
            lines = parallel_job_running_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.parallel_job.running.out",
                    "lines": lines,
                    "value": "/cmb/g3/chenjing/WRF-REPS/operation/gefs_ruc/2017052112/p02/tmpdir_wrf/"
                             "printout/wrf_6173907.out",
                    "label": "Out",
                },
                {
                    "name": "llq.parallel_job.running.err",
                    "lines": lines,
                    "value": "/cmb/g3/chenjing/WRF-REPS/operation/gefs_ruc/2017052112/p02/tmpdir_wrf/"
                             "printout/wrf_6173907.err",
                    "label": "Err",
                }
            ])

        for a_test_case in test_case_list:
            check_method(a_test_case)

    def check_table_record_parser(self, test_case):
        line = test_case["line"]
        value = test_case["value"]
        name = test_case["name"]
        begin_pos = test_case["record_parser"]["args"]["begin_pos"]
        end_pos = test_case["record_parser"]["args"]["end_pos"]

        parser = record_parser.TableRecordParser(begin_pos, end_pos)
        parser_value = parser.parse(line)
        self.assertEqual(parser_value, value)
        print("Test passed:", name)

    def test_table_record_parser(self):
        check_method = self.check_table_record_parser
        simple_default_query_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/default_query/llq/simple.default.query.txt"
        )

        test_case_list = []
        with open(simple_default_query_file_path) as simple_default_query_file:
            lines = simple_default_query_file.readlines()
            test_case_list.extend([
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 0,
                            "end_pos": 24
                        }
                    },
                    "value": "cma19n04.4192420.0"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 25,
                            "end_pos": 35
                        }
                    },
                    "value": "nwp"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 36,
                            "end_pos": 47
                        }
                    },
                    "value": "1/13 03:38"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 48,
                            "end_pos": 50
                        }
                    },
                    "value": "R"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 51,
                            "end_pos": 54
                        }
                    },
                    "value": "100"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 55,
                            "end_pos": 67
                        }
                    },
                    "value": "operation"
                },
                {
                    "name": "llq.default_query.simple.1",
                    "line": lines[2].strip(),
                    "record_parser": {
                        "args": {
                            "begin_pos": 68,
                            "end_pos": 79
                        }
                    },
                    "value": "cma02n05"
                }
            ])
        for a_test_case in test_case_list:
            check_method(a_test_case)



# coding=utf-8
from .query_property import QueryProperty


class QueryItem(object):
    def __init__(self):
        self.props = list()

    def to_dict(self):
        result = dict()
        result['props'] = list()
        for prop in self.props:
            result['props'].append(prop.to_dict())
        return result

    @classmethod
    def build_from_category_list(cls, record, category_list):
        item = QueryItem()

        for a_category in category_list:
            p = QueryProperty.build_from_category(record, a_category)
            item.props.append(p)

        return item



# coding=utf-8
from nwpc_hpc_model.loadleveler.filter import Filter


def test_filter():
    a_filter = Filter()
    job_items = [
        {},
        {}
    ]

    target_job_items = a_filter.filter(job_items)

    assert job_items == target_job_items



# coding: utf-8
from pathlib import Path

from nwpc_hpc_model.workload.slurm import SlurmQueryModel, SlurmQueryCategoryList
from nwpc_hpc_model.workload import QueryCategory, record_parser, value_saver


def test_empty_build_from_table_category_list():
    category_list = SlurmQueryCategoryList()
    category_list.extend([
        QueryCategory("sinfo.partition", "Partition", "PARTITION",
                      record_parser.TokenRecordParser,  (-1,),
                      value_saver.StringSaver, ())])

    model = SlurmQueryModel.build_from_table_category_list([], category_list)
    assert model is None

    model = SlurmQueryModel.build_from_table_category_list(
        ['PARTITION AVAIL  TIMELIMIT  NODES  STATE NODELIST',
         ''], category_list
    )
    assert len(model.items) == 0

    model = SlurmQueryModel.build_from_table_category_list(
        ['PARTITION AVAIL  TIMELIMIT  NODES  STATE NODELIST'],
        category_list
    )
    assert len(model.items) == 0



# coding: utf-8
class Filter(object):
    def __init__(self):
        self.conditions = list()
        pass

    def filter(self, job_items):
        target_job_items = list()
        for a_job_item in job_items:
            is_fit = True
            for a_condition in self.conditions:
                if not a_condition.is_fit(a_job_item):
                    is_fit = False
                    break
            if is_fit:
                target_job_items.append(a_job_item)
        return target_job_items



# coding=utf-8


class RecordParser(object):
    def __init__(self):
        pass

    def parse(self, record):
        pass


class TableRecordParser(RecordParser):
    def __init__(self, begin_pos, end_pos):
        RecordParser.__init__(self)
        self.begin_pos = int(begin_pos)
        self.end_pos = int(end_pos)

    def parse(self, record):
        return record[self.begin_pos:self.end_pos].strip()


class TokenRecordParser(RecordParser):
    def __init__(self, index=-1, sep=None):
        RecordParser.__init__(self)
        self.index = index
        self.sep = sep

    def parse(self, record):
        if self.sep is None:
            tokens = record.split()
        else:
            tokens = record.split(self.sep)
        return tokens[self.index]



# coding=utf-8
import unittest
from unittest import mock
import os
import importlib
import datetime

from nwpc_hpc_model.loadleveler import query_item, query_property, query_category, record_parser, value_saver


class TestQueryItem(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_to_dict(self):
        category_list = query_category.QueryCategoryList()

        category_list.extend([
            query_category.QueryCategory("llq.id", "Id", "Job Step Id",
                                         record_parser.DetailLabelParser, ("Job Step Id",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.owner", "Owner", "Owner",
                                         record_parser.DetailLabelParser, ("Owner",),
                                         value_saver.StringSaver, ()),
        ])

        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        with open(serial_job_running_file_path) as serial_job_running_file:
            record = serial_job_running_file.readlines()
            item = query_item.QueryItem.build_from_category_list(record, category_list)
            item_dict = item.to_dict()
            self.assertTrue('props' in item_dict)
            self.assertIsInstance(item_dict['props'], list)
            self.assertEqual(len(item_dict['props']), 2)

    def check_build_from_category(self, test_case):
        record = test_case['record']
        category_list = test_case['category_list']
        properties = test_case['properties']

        item = query_item.QueryItem.build_from_category_list(record, category_list)

        for a_property in properties:
            category_id = a_property['category_id']
            value = a_property['value']
            text = a_property['text']
            data = a_property['data']

            index = category_list.index_from_id(category_id)
            if index == -1:
                self.fail("can't find category with id: "+category_id)
            p = item.props[index]
            self.assertEqual(p.map['value'], value)
            self.assertEqual(p.map['text'], text)
            self.assertEqual(p.map['data'], data)

    def test_build_from_category_list(self):
        check_method = self.check_build_from_category
        category_list = query_category.QueryCategoryList()

        category_list.extend([
            query_category.QueryCategory("llq.id", "Id", "Job Step Id",
                                         record_parser.DetailLabelParser, ("Job Step Id",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.owner", "Owner", "Owner",
                                         record_parser.DetailLabelParser, ("Owner",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.queue_full_date", "Queue Date", "Queue Date",
                                         record_parser.DetailLabelParser, ("Queue Date",),
                                         value_saver.FullDateSaver, ()),
            query_category.QueryCategory("llq.job_script", "Job Script", "Cmd",
                                         record_parser.DetailLabelParser, ("Cmd",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.status", "Status", "Status",
                                         record_parser.DetailLabelParser, ("Status",),
                                         value_saver.JobStatusSaver, ())
        ])

        test_case_list = list()

        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        with open(serial_job_running_file_path) as serial_job_running_file:
            lines = serial_job_running_file.readlines()

            test_case_list.append({
                'record': lines,
                'category_list': category_list,
                'properties': [
                    {
                        'category_id': 'llq.id',
                        'value': 'cma20n04.2882680.0',
                        'text': 'cma20n04.2882680.0',
                        'data': 'cma20n04.2882680.0'
                    },
                    {
                        'category_id': "llq.queue_full_date",
                        'value': 'Thu Sep  8 00:09:02 2016',
                        'text': '09/08 00:09',
                        'data': datetime.datetime.strptime("2016/09/08 00:09:02", "%Y/%m/%d %H:%M:%S")
                    },
                    {
                        'category_id': "llq.status",
                        'value': 'Running',
                        'text': 'R',
                        'data': 'R'
                    },
                ]

            })

        for a_test_case in test_case_list:
            check_method(a_test_case)


from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='nwpc-hpc-model',

    version='0.2.0',

    description='A collection of models for HPC used in NWPC.',
    long_description=long_description,

    url='https://github.com/perillaroc/nwpc-hpc-model',

    author='perillaroc',
    author_email='perillaroc@gmail.com',

    license='GPL-3.0',

    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6'
    ],

    keywords='nwpc hpc model',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=[],

    extras_require={
        'test': ['pytest'],
    },

    entry_points={}
)



# coding=utf-8
import unittest
from unittest import mock
import os
import importlib
import datetime

from nwpc_hpc_model.loadleveler import query_property, query_category, record_parser, value_saver


class TestQueryProperty(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_to_dict(self):
        # TODO: test_to_dict
        pass

    def check_query_item(self, test_case):
        record = test_case["record"]
        category = test_case["category"]
        value = test_case["value"]
        text = test_case["text"]
        data = test_case["data"]

        item = query_property.QueryProperty.build_from_category(
            record=record,
            category=category
        )
        self.assertIsInstance(item, query_property.QueryProperty)
        self.assertEqual(item.map['value'], value)
        self.assertEqual(item.map['text'], text)
        self.assertEqual(item.map['data'], data)
        self.assertEqual(item.category, category)

    def test_create_query_property(self):
        check_method = self.check_query_item
        category_list = query_category.QueryCategoryList()

        category_list.extend([
            query_category.QueryCategory("llq.id", "Id", "Job Step Id",
                                         record_parser.DetailLabelParser, ("Job Step Id",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.owner", "Owner", "Owner",
                                         record_parser.DetailLabelParser, ("Owner",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.queue_full_date", "Queue Date", "Queue Date",
                                         record_parser.DetailLabelParser, ("Queue Date",),
                                         value_saver.FullDateSaver, ()),
            query_category.QueryCategory("llq.job_script", "Job Script", "Cmd",
                                         record_parser.DetailLabelParser, ("Cmd",),
                                         value_saver.StringSaver, ()),
            query_category.QueryCategory("llq.status", "Status", "Status",
                                         record_parser.DetailLabelParser, ("Status",),
                                         value_saver.JobStatusSaver, ())
        ])

        test_case_list = list()

        serial_job_running_file_path = os.path.join(
            os.path.dirname(__file__),
            "../data/detail_query/llq/serial_job_running.txt"
        )
        with open(serial_job_running_file_path) as serial_job_running_file:
            lines = serial_job_running_file.readlines()

            test_case_list.extend([
                {
                    'record': lines,
                    'category': category_list.category_from_id("llq.id"),

                    'value': 'cma20n04.2882680.0',
                    'text': 'cma20n04.2882680.0',
                    'data': 'cma20n04.2882680.0'
                },
                {
                    'record': lines,
                    'category': category_list.category_from_id("llq.queue_full_date"),

                    'value': 'Thu Sep  8 00:09:02 2016',
                    'text': '09/08 00:09',
                    'data': datetime.datetime.strptime("2016/09/08 00:09:02", "%Y/%m/%d %H:%M:%S")
                },
                {
                    'record': lines,
                    'category': category_list.category_from_id("llq.status"),

                    'value': 'Running',
                    'text': 'R',
                    'data': 'R'
                },
            ])

        for a_test_case in test_case_list:
            check_method(a_test_case)



# coding=utf-8
import copy


class QueryProperty(object):
    def __init__(self):
        self.map = dict()
        self.category = None

    def to_dict(self):
        result = copy.deepcopy(self.map)
        if self.category is not None:
            result['id'] = self.category.id
        return result

    @staticmethod
    def build_from_category(record, category):
        prop = QueryProperty()

        prop.category = category
        value = category.record_parser.parse(record)
        category.value_saver.set_item_value(prop, value)
        return prop
